/**
 * Detects the image links from the given link pattern and make a simple viewer
 * for them
 *
 * An image container should similar to this:
 *   <div class="attach-center"><a href="*.jpg">#-thumb.jpg</a><p>caption</p></div>
 * where the attach center contains the thumbnail which is in an anchor which href attribute
 * is the original image's link and a <p></p> which contains the caption of the image.
 * the caption is not required
 *
 * @class Drupal.PictureViewer
 * @constructor
 * @author Koszti Lajos http://ajnasz.hu
 *
 * @param {String} linkPattern jquery style pattern to collect the links
 *   which target is an image, what you want to display on the overlay
 * @param {Boolean} animate if true, the resize for the image will be animated
 */
Drupal.PictureViewer = function(linkPattern, animate) {
  this.body = $(document.body);
  this.linkPattern = linkPattern || 'div#main div.attach-center a';
  this.addOverlay();
  this.addContainer();
  this.getImageLinks();
  this.animate = animate || false;
};
Drupal.PictureViewer.prototype = {
  imageLinks: null,
  animate: false,
  overlay: null,
  container: null,
  img: null,
  caption: null,
  clickedLink: null,
  next: null,
  prev: null,
  /**
   * add the caption, the viewlink, the next the previous images links to the container
   * @param {Object} link a link object
   */
  addCaption: function(link) {
    var caption = this.getCaption(link);
    this.caption = document.createElement('div');
    this.caption.id = 'img-caption';
    this.caption = $(this.caption);
    this.caption.append(this.createViewLink(link.href));
    if(caption) {
      this.caption.append(caption.text());
      this.container.append(this.caption);
    }
    var nextPrev = this.getPrevNextLinks();
    if(nextPrev.previous) {
      this.addPrevious(nextPrev.previous);
    }
    if(nextPrev.next) {
      this.addNext(nextPrev.next);
    }
    this.addCloseButton();
  },
  addCloseButton: function() {
    this.close = $(new Image());
    this.close.attr('src', '/' + Drupal.settings.picture_viewer.module_url + '/close.png');
    var _this = this;
    this.close.click(function(){
      _this.hideViewer('button');
    });
    this.close.addClass('close');
    this.container.append(this.close);
  },
  /**
   * add the container to the document.body
   */
  addContainer: function() {
    if(!this.container) {
      this.container = document.createElement('div');
      this.container.id = 'img-container';
      this.container = $(this.container);
      this.body.append(this.container);
      this.container.css({left: (this.body.width()/2) - (this.container.width()/2) + 'px'});
      this.container.hover(function(){
        $(this).addClass('hover');
      },
      function(){
        $(this).removeClass('hover');
      });
      this.container.hide();
    }
  },
  /**
   * add the image to the container
   * @param {Object} link a link object
   */
  addImage: function(link) {
    var src = link.href;
    this.img = $(new Image());
    var _this = this;
    this.img.attr('src', src).load(function(){
      _this.container.append(_this.img);
      _this.img.css(_this.getImageStyle());
      var containerStyle = _this.getContainerStyle();
      _this.img.hide();
      if(_this.animate &&
          (containerStyle.width != _this.container.css('width') &&
          containerStyle.height != _this.container.css('height') &&
          containerStyle.top != _this.container.css('top') &&
          containerStyle.left != _this.container.css('left'))
       ) {
        _this.container.animate(containerStyle, {duration: 1000, complete: function(){
          _this.img.show();
          _this.container.removeClass('loading');
          _this.addCaption(link);
        }});
      } else {
        _this.container.css(containerStyle);
        _this.img.show();
        _this.container.removeClass('loading');
        _this.addCaption(link);
      }
    });
    this.img.error(function() {
        _this.container.removeClass('loading');
        _this.addCaption(link);
    });
  },
  /**
   * add the next link to the container
   * @param {Object} link a link object
   */
  addNext: function(link) {
    var img = $(new Image());
    img.attr('src', '/' + Drupal.settings.picture_viewer.module_url + '/next.png');
    this.next = $(document.createElement('a'));
    var _this = this;
    this.next.click(function() {
      _this.showImage(link.image);
      if(typeof gaLog != 'undefined') console.log('view picture', 'next', link.image.href);
      if(pageTracker) {
        pageTracker._trackEvent('view picture', 'next', link.image.href);
      }
    });
    this.next.addClass('next-prev next');
    this.next.css({height: (_this.container.height() - 40) + 'px'})
    //this.next.append(Drupal.t('next'));
    this.next.append(img);
    this.container.append(this.next);
  },
  /**
   * add the overlay to the document body
   */
  addOverlay: function() {
    if(!this.overlay) {
      this.overlay = document.createElement('div');
      this.overlay.id = 'img-overlay';
      this.overlay = $(this.overlay);
      this.body.append(this.overlay);
      this.overlay.hide();
      // hide the viewer when clicing on the overlay
      var _this = this;
      this.overlay.click(function(){
        _this.hideViewer('overlay');
      });
    }
  },
  /**
   * add the previous link to the container
   * @param {Object} link a link object
   */
  addPrevious: function(link) {
    var img = $(new Image());
    img.attr('src', '/' + Drupal.settings.picture_viewer.module_url + '/previous.png');
    this.prev = $(document.createElement('a'));
    var _this = this;
    this.prev.click(function() {
      _this.showImage(link.image);
      if(typeof gaLog != 'undefined') console.log('view picture', 'previous', link.image.href);
      if(pageTracker) {
        pageTracker._trackEvent('view picture', 'previous', link.image.href);
      }
    });
    this.prev.addClass('next-prev previous');
    this.prev.css({height: (_this.container.height() - 20) + 'px'})
    // this.prev.append(Drupal.t('previous'));
    this.prev.append(img);
    this.container.append(this.prev);
  },
  /**
   * add the view link to the caption
   * @param {String} href the src of the image
   * @return a jQuery anchor object
   * @type Object
   */
  createViewLink: function(href) {
    var dl = document.createElement('a');
    dl.href = href;
    dl.title = Drupal.t('view this photo in its original size');
    dl = $(dl);
    dl.click(function(event){
      if(typeof gaLog != 'undefined') {console.log('view picture', 'download', this.href);}
      if(typeof pageTracker != 'undefined') {
        pageTracker._trackEvent('view picture', 'download', this.href);
      }
    })
    dl.addClass('img-view-link');
    dl.append(Drupal.t('download'));
    return dl;
  },
  /**
   * get the caption of the image from it's link.
   * @param {Object} link a link object
   * @return the caption of the image
   * @type Object a jQuery paragraph object
   */
  getCaption: function(link) {
    var caption = $(link.parentNode).children('p');
    if(caption.length) {
      caption = $(caption[0]);
    } else {
      caption = null;
    }
    return caption;
  },
  /**
   * hide the viewer
   */
  hideViewer: function(type) {
    this.resetContainer();
    this.overlay.hide();
    this.container.hide();
    if(typeof gaLog != 'undefined') console.log('view picture', 'close', type);
    if(pageTracker) {
      pageTracker._trackEvent('view picture', 'close', type);
    }
  },
  /**
   * compute the style for the image container before a new image gets displayed
   * @return the styles of the container
   * @type Object
   */
  getContainerStyle: function() {
    return {
      width: (this.img.width()) + 'px',
      height: (this.img.height() + 20) + 'px', // 20 is the caption size
      left: (this.body.width()/2) - (this.img.width()/2) + 'px'
    };
  },
  /**
   * collects the links from the page which are links to an image
   */
  getImageLinks: function() {
    var links = $(this.linkPattern), imageLinks = {length:0}, _this = this;
    links.each(function(i) {
      if(/(:?png|jpg|gif|jpeg)$/i.test(this.href)) {
        $(this).click(function() {
          _this.showImage(this);
          if(typeof gaLog != 'undefined') console.log('view picture', 'start', this.href);
          if(pageTracker) {
            pageTracker._trackEvent('view picture', 'start', this.href);
          }
          return false;
        })
        imageLinks[this] = {
          index: imageLinks.length,
          image: this
        };
        imageLinks.length++;
      }
    });
    if(imageLinks.length) {
      this.imageLinks = imageLinks;
    }
  },
  /**
   * compute the style for a new image to fit in the window
   * @return the style for the image {height: xxpx, width: xxpx}
   * @type Object
   */
  getImageStyle: function() {
    var imgStyle = {}, imgH = this.img.height(), imgW = this.img.width(), wh = imgW/imgH, hw = imgH/imgW, margin = 60, newHeight, newWidth;
    if(this.img.height() > ($(window).height() - margin)) {
      newHeight = ($(window).height() - margin);
      newWidth  = newHeight*wh;
      imgStyle.height = newHeight + 'px';
      imgStyle.width = newWidth + 'px';
      if(newWidth > (this.body.width() - margin)) {
        newWidth = (this.body.width() - margin);
        newHeight = newWidth*hw;
        imgStyle.width = newWidth + 'px';
        imgStyle.height = newHeight + 'px';
      }
    } else if(this.img.width() > (this.body.width() - margin)) {
      newWidth = (this.body.width() - margin);
      newHeight = newWidth*hw;
      imgStyle.width =  newWidth + 'px';
      imgStyle.height = newHeight + 'px';
      if(newHeight > ($(window).height() - margin)) {
        newHeight = ($(window).height() - margin);
        newWidth  = newHeight*wh;
        imgStyle.height = newHeight + 'px';
        imgStyle.width = newWidth + 'px';
      }
    }
    return imgStyle;
  },
  /**
   * search for the the available next and previous image links
   * @return an object which contains the previous, the next and the current image links
   *  {next: {Linkobject|null}, previous: {Linkobject|null}, current: Linkobject}
   * @type Object
   */
  getPrevNextLinks: function() {
    if(this.clickedLink) {
      var current = this.imageLinks[this.clickedLink], previous = null, next = null, currentFound = false, i;
      if(current.index != 0 && current.index < this.imageLinks.length - 1) {
        for(i in this.imageLinks) {
          if(currentFound) {
            next = this.imageLinks[i];
            break;
          }
          if(this.imageLinks[i].index == current.index) {
            currentFound = true;
          } else {
            previous = this.imageLinks[i];
          }
        }
      } else if(current.index == 0) {
        for(i in this.imageLinks) {
          if(currentFound) {
            next = this.imageLinks[i];
            break;
          }
          if(this.imageLinks[i].index == current.index) {
            currentFound = true;
          }
        }
      } else {
        for(i in this.imageLinks) {
          if(this.imageLinks[i].index == current.index) {
            break;
          } else {
            previous = this.imageLinks[i];
          }
        }
      }
    }
    return {
      current: current,
      next: next,
      previous: previous
    };
  },
  /**
   * remove all element from the image container
   */
  resetContainer: function() {
    if(this.img) {
      this.img.remove();
    }
    if(this.caption) {
      this.caption.remove();
    }
    if(this.prev) {
      this.prev.remove();
      this.prev = null;
    }
    if(this.next) {
      this.next.remove();
      this.next = null;
    }
    if(this.close) {
      this.close.remove();
      this.close = null;
    }
  },
  /**
   * show the overlay, the image container and append the new image to the container
   * @param {Object} link a link object
   */
  showImage: function(link) {
    this.resetContainer();
    var scrollTop = $(document).scrollTop();
    this.clickedLink = link;
    this.overlay.css({height: this.body.height() + 'px', width: this.body.width() + 'px'});
    this.container.addClass('loading');
    this.container.css({top: (scrollTop + 15) + 'px'});
    this.overlay.show();
    this.container.show();
    this.addImage(link);
  }
};
$(document).ready(function(){
  new Drupal.PictureViewer(null, true);
});
